## $Id: interfaces.py 9512 2012-11-02 16:35:06Z henrik $
##
## Copyright (C) 2011 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
from zope.interface import Attribute
from zope import schema
from waeup.kofa.university.vocabularies import ContextualDictSourceFactoryBase
from waeup.kofa.interfaces import (
    IKofaObject, SimpleKofaVocabulary, academic_sessions_vocab)
from waeup.kofa.interfaces import MessageFactory as _

payment_states = SimpleKofaVocabulary(
    (_('Not yet paid'),'unpaid'),
    (_('Paid'),'paid'),
    (_('Failed'),'failed'),
    )

class PaymentCategorySource(ContextualDictSourceFactoryBase):
    """A application category source delivers all special handling categories
    provided for accommodation booking.
    """
    #: name of dict to deliver from kofa utils.
    DICT_NAME = 'PAYMENT_CATEGORIES'

class IPaymentsContainer(IKofaObject):
    """A container for all kind of payment objects.

    """

class IPayment(IKofaObject):
    """A base representation of payments.

    """
    p_id = Attribute('Payment identifier.')

    p_category = schema.Choice(
        title = _(u'Payment Category'),
        default = u'schoolfee',
        source = PaymentCategorySource(),
        required = True,
        )

    p_item = schema.TextLine(
        title = _(u'Payment Item'),
        default = None,
        required = False,
        )

    p_session = schema.Choice(
        title = _(u'Payment Session'),
        source = academic_sessions_vocab,
        required = True,
        )

    p_state = schema.Choice(
        title = _(u'Payment State'),
        default = u'unpaid',
        vocabulary = payment_states,
        required = True,
        )

    creation_date = schema.Datetime(
        title = _(u'Ticket Creation Date'),
        readonly = False,
        required = False,
        )

    payment_date = schema.Datetime(
        title = _(u'Payment Date'),
        required = False,
        readonly = False,
        )

    amount_auth = schema.Float(
        title = _(u'Amount Authorized'),
        default = 0.0,
        required = True,
        readonly = False,
        )

class IOnlinePayment(IPayment):
    """A payment via payment gateways.

    """

    ac = schema.TextLine(
        title = _(u'Activation Code'),
        default = None,
        required = False,
        readonly = False,
        )

    r_amount_approved = schema.Float(
        title = _(u'Response Amount Approved'),
        default = 0.0,
        required = False,
        readonly = False,
        )

    r_code = schema.TextLine(
        title = _(u'Response Code'),
        default = None,
        required = False,
        readonly = False,
        )

    r_desc = schema.TextLine(
        title = _(u'Response Description'),
        default = None,
        required = False,
        readonly = False,
        )

    def approve():
        "Approve an online payment and set to paid."

class IPaymentWebservice(IKofaObject):
    """An interface for a webservice.

    """